<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 * @license		http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Backend;

/**
 * Class file
 * TableContent
 * Provide methods for table tl_content in do='revolutionslider'
 */
class TableContent extends \Backend
{	
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser','User');
	}

	/**
	 * Check permissions to edit table tl_content
	 */
	public function checkPermission()
	{
		if ($this->User->isAdmin)
		{
			return;
		}

		// Set the root IDs
		if (!is_array($this->User->revolutionslider_slides) || empty($this->User->revolutionslider_slides))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->revolutionslider_slides;
		}

		// Check the current action
		switch (\Input::get('act'))
		{
			case 'paste':
				// Allow
				break;

			case '': // empty
			case 'create':
			case 'select':
				// Check access to the sliders item
				if (!$this->checkAccessToElement(CURRENT_ID, $root, true))
				{
					$this->redirect('contao/main.php?act=error');
				}
				break;

			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
			case 'cutAll':
			case 'copyAll':
				// Check access to the parent element if a content element is moved
				if (($objInput->get('act') == 'cutAll' || $objInput->get('act') == 'copyAll') && !$this->checkAccessToElement(Input::get('pid'), $root, (Input::get('mode') == 2)))
				{
					$this->redirect('contao/main.php?act=error');
				}

				$objCes = \Database::getInstance()->prepare("SELECT id FROM tl_content WHERE ptable='tl_revolutionslider_slides' AND pid=?")
										 ->execute(CURRENT_ID);

				$session = \Session::getInstance()->getData();
				$session['CURRENT']['IDS'] = array_intersect($session['CURRENT']['IDS'], $objCes->fetchEach('id'));
				\Session::getInstance()->setData($session);
				break;

			case 'cut':
			case 'copy':
				// Check access to the parent element if a content element is moved
				if (!$this->checkAccessToElement($objInput->get('pid'), $root, ($objInput->get('mode') == 2)))
				{
					$this->redirect('contao/main.php?act=error');
				}
				// NO BREAK STATEMENT HERE

			default:
				// Check access to the content element
				if (!$this->checkAccessToElement(\Input::get('id'), $root))
				{
					$this->redirect('contao/main.php?act=error');
				}
				break;
		}
	}


	/**
	 * Check access to a particular content element
	 * @param integer
	 * @param array
	 * @param boolean
	 * @return boolean
	 */
	protected function checkAccessToElement($id, $root, $blnIsPid=false)
	{
		if ($blnIsPid)
		{
			$objArchive = \Database::getInstance()->prepare("SELECT a.id, n.id AS nid FROM tl_revolutionslider_slides n, tl_revolutionslider a WHERE n.id=? AND n.pid=a.id")
										 ->limit(1)
										 ->execute($id);
		}
		else
		{
			$objArchive = \Database::getInstance()->prepare("SELECT a.id, n.id AS nid FROM tl_content c, tl_revolutionslider_slides n, tl_revolutionslider a WHERE c.id=? AND c.pid=n.id AND n.pid=a.id")
										 ->limit(1)
										 ->execute($id);
		}

		// Invalid ID
		if ($objArchive->numRows < 1)
		{
			$this->log('Invalid slides content element ID ' . $id, 'tl_revolutionslider_slides checkAccessToElement()', TL_ERROR);
			return false;
		}
		// The slide is not mounted
		if (!in_array($objArchive->nid, $root))
		{
			$this->log('Not enough permissions to modify slide ID ' . $objArchive->nid . ' in slider ID ' . $objArchive->id, 'tl_revolutionslider_slides checkAccessToElement()', TL_ERROR);
			return false;
		}

		return true;
	}
	
	
	/**
	 * Return all slider ids as array
	 * @param object
	 * @return array
	 */
	public function getSliders(\DataContainer $objDC)
	{
		$objDatabase = \Database::getInstance();
		$objResult = $objDatabase->execute("SELECT * FROM tl_revolutionslider");
						
		if($objResult->numRows < 1)
		{
			return null;
		}
		
		$arrReturn = array();
		while($objResult->next())
		{
			$arrReturn[$objResult->id] = $objResult->title . ' (id:'.$objResult->id.')';
		}
		
		return $arrReturn;
	}
	
	/**
	 * Update the revolution slider table with source information
	 * @param object
	 */
	public function updateSliderRecord(\DataContainer $objDC)
	{
		$objInput = \Input::getInstance();
		$objDatabase = \Database::getInstance();
		
		$objDatabase->prepare("UPDATE tl_revolutionslider %s WHERE id=?")
					->set(array('pid'=>$objDC->id, 'source'=>$objDC->table))
					->execute($objDC->activeRecord->revolutionslider);
	}
	
	/**
	 * Get all slides of the current slider an return as array
	 * @param object
	 * @return array
	 */
	public function getSlides(\DataContainer $objDC)
	{
		$objDatabase = \Database::getInstance();
		
		$objSlides = $objDatabase->prepare(
			"SELECT * FROM tl_revolutionslider_slides WHERE pid=
			(
				SELECT pid FROM tl_revolutionslider_slides WHERE id=?
			) AND id!=? ORDER BY sorting"
		)->execute($objDC->activeRecord->pid,$objDC->activeRecord->pid);
		
		if($objSlides->numRows < 1)
		{
			return array();
		}
		
		$arrReturn = array();
		while($objSlides->next())
		{
			$arrReturn[$objSlides->id] = $objSlides->title . ' (id:' .$objSlides->id.')';
		}
		
		return $arrReturn;
	}
	
	/**
	 * Modify the palettes depending on selected video type
	 * @param object
	 * @return object
	 */
	public function modifyPalettes(\DataContainer $objDC)
	{
		if(\Input::getInstance()->get('act') != 'edit')
		{
			return $objDC;
		}
		
		$objDatabase = \Database::getInstance();
		$objActiveRecord = $objDatabase->prepare("SELECT * FROM ".$objDC->table." WHERE id=?")->limit(1)->execute($objDC->id);
		
		if($objActiveRecord->type == 'revolutionslider_video')
		{
			switch($objActiveRecord->revolutionslider_videoType)
			{
				case 'local':
					$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video'] = '{type_legend},type,headline;{select_legend},revolutionslider_videoType,multiSRC,revolutionslider_data_autoplay,revolutionslider_data_nextslideatend;{protected_legend:hide},protected;{expert_legend:hide},guests,invisible,cssID,space';
					break;
				case 'youtube':
				case 'vimeo':
				case 'external':
					$GLOBALS['TL_DCA']['tl_content']['palettes']['revolutionslider_video'] = '{type_legend},type,headline;{select_legend},revolutionslider_videoType,url,revolutionslider_data_autoplay,revolutionslider_data_nextslideatend;{protected_legend:hide},protected;{expert_legend:hide},guests,invisible,cssID,space';
					break;
				default: 
					
					break;
			}
			
			#fix: overwrite default contao fields only when revolutionslider_video is active	
			// singleSRC
			$GLOBALS['TL_DCA']['tl_content']['fields']['singleSRC']['eval']['mandatory'] = false;
			$GLOBALS['TL_DCA']['tl_content']['fields']['multiSRC']['eval']['mandatory'] = false;
			
			// url
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['label'] = &$GLOBALS['TL_LANG']['tl_content']['revolutionslider']['url'];
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['inputType'] = 'text';
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['wizard'] = null;
			$GLOBALS['TL_DCA']['tl_content']['fields']['url']['eval'] = array('mandatory'=>false);
		}
		
		return $objDC;
	}
	
}
