<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 * @license		http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Backend;

/**
 * Imports
 */
use \Input;
use \Image;

/**
 * Class file
 * TableRevolutionSliderSlides
 * Provide methods for table tl_revolutionslider_slides
 */
class TableRevolutionSliderSlides extends \Backend
{	
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser','User');
	}
	
	/**
	 * Load styles
	 * @param object
	 */
	public function loadStyles(\DataContainer $objDC)
	{
		$GLOBALS['TL_CSS'][] = 'system/modules/revolutionslider/assets/css/be_style.css';
	}
	
	/**
	 * Check permissions
	 */
	public function checkPermission()
	{
		if ($this->User->isAdmin)
		{
			return;
		}

		// Set root IDs
		if (!is_array($this->User->revolutionslider_slides) || empty($this->User->revolutionslider_slides))
		{
			$root = array(0);
		}
		else
		{
			$root = $this->User->revolutionslider_slides;
		}

		$GLOBALS['TL_DCA']['tl_revolutionslider_slides']['list']['sorting']['root'] = $root;

		// Check permissions to add slides
		if (!$this->User->hasAccess('create', 'revolutionslider_slidesp'))
		{
			$GLOBALS['TL_DCA']['tl_revolutionslider_slides']['config']['closed'] = true;
		}
		
		// Check current action
		switch (\Input::get('act'))
		{
			case 'create':
			case 'select':
			case 'show':
				// Allow
				break;
			
			case 'cut':
			case 'paste':
				if (!$this->User->hasAccess('cut', 'revolutionslider_slidesp'))
				{
					$this->redirect('contao/main.php?act=error');
				}
				break;
			case 'copy':
				if (!$this->User->hasAccess('cut', 'revolutionslider_slidesp'))
				{
					$this->redirect('contao/main.php?act=error');
				}
				break;
			case 'edit':
			case 'toggle':
				// Dynamically add the record to the user profile
				if (!in_array(Input::get('id'), $root))
				{
					$arrNew = $this->Session->get('new_records');

					if (is_array($arrNew['tl_revolutionslider_slides']) && in_array(Input::get('id'), $arrNew['tl_revolutionslider_slides']))
					{
						// Add permissions on user level
						if ($this->User->inherit == 'custom' || !$this->User->groups[0])
						{
							$objUser = $this->Database->prepare("SELECT revolutionslider_slides, revolutionslider_slidesp FROM tl_user WHERE id=?")
													   ->limit(1)
													   ->execute($this->User->id);

							$arrSlidesp = deserialize($objUser->revolutionslider_slidesp);

							if (is_array($arrSlidesp) && in_array('create', $arrSlidesp))
							{
								$arrSliders = deserialize($objUser->revolutionslider_slides);
								$arrSliders[] = Input::get('id');

								$this->Database->prepare("UPDATE tl_user SET revolutionslider_slides=? WHERE id=?")
											   ->execute(serialize($arrSliders), $this->User->id);
							}
						}

						// Add permissions on group level
						elseif ($this->User->groups[0] > 0)
						{
							$objGroup = $this->Database->prepare("SELECT revolutionslider_slides, revolutionslider_slidesp FROM tl_user_group WHERE id=?")
													   ->limit(1)
													   ->execute($this->User->groups[0]);

							$arrSliderp = deserialize($objGroup->revolutionslider_slidesp);

							if (is_array($arrSliderp) && in_array('create', $arrSliderp))
							{
								$arrSliders = deserialize($objGroup->revolutionslider_slides);
								$arrSliders[] = Input::get('id');

								$this->Database->prepare("UPDATE tl_user_group SET revolutionslider_slides=? WHERE id=?")
											   ->execute(serialize($arrSliders), $this->User->groups[0]);
							}
						}

						// Add new element to the user object
						$root[] = Input::get('id');
						$this->User->revolutionslider_slides = $root;
					}
				}
				// No break;
			case 'delete':
				if (!$this->User->hasAccess('delete', 'revolutionslider_slidesp'))
				{
					$this->log('Not enough permissions to '.Input::get('act').' slider ID "'.Input::get('id').'"', 'tl_revolutionslider_slides checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;
			case 'editAll':
			case 'deleteAll':
			case 'overrideAll':
				$session = $this->Session->getData();
				if (Input::get('act') == 'deleteAll' && !$this->User->hasAccess('delete', 'revolutionslider_slidesp'))
				{
					$session['CURRENT']['IDS'] = array();
				}
				else
				{
					$session['CURRENT']['IDS'] = array_intersect($session['CURRENT']['IDS'], $root);
				}
				$this->Session->setData($session);
				break;

			default:
				if (strlen(Input::get('act')))
				{
					$this->log('Not enough permissions to '.Input::get('act').' slider', 'tl_revolutionslider_slides checkPermission', TL_ERROR);
					$this->redirect('contao/main.php?act=error');
				}
				break;
		}
	}
	
	
	/**
	 * List slides
	 * @param array
	 * @return string
	 */
	public function listSlides($arrRow)
	{
		return '<div class="tl_content_left">' . $arrRow['title'] . '</div>';
	}
	
	/**
	 * Return the edit button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function edit($row, $href, $label, $title, $icon, $attributes)
	{
		return '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.$this->generateImage($icon, $label).'</a> ';
	}
	
	/**
	 * Return the edit header button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function editHeader($row, $href, $label, $title, $icon, $attributes)
	{
		$strButton = '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.Image::getHtml($icon, $label).'</a> ';
		if($this->User->isAdmin )
		{
			return $strButton;	
		}
		elseif($this->User->hasAccess('create', 'revolutionslider_slidesp') && count(preg_grep('/^tl_revolutionslider_slides::/', $this->User->alexf)) > 0 )
		{
			return $strButton;			
		}
		else
		{
			return '';
		}
	}

	/**
	 * Return the copy category button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function copyButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('create', 'revolutionslider_slidesp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.Image::getHtml($icon, $label).'</a> ' : Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}

	/**
	 * Return the delete archive button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function deleteButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('delete', 'revolutionslider_slidesp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.Image::getHtml($icon, $label).'</a> ' : Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}
	
	/**
	 * Return the copy category button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function cutButton($row, $href, $label, $title, $icon, $attributes)
	{
		return ($this->User->isAdmin || $this->User->hasAccess('cut', 'revolutionslider_slidesp')) ? '<a href="'.$this->addToUrl($href.'&amp;id='.$row['id']).'" title="'.specialchars($title).'"'.$attributes.'>'.Image::getHtml($icon, $label).'</a> ' : Image::getHtml(preg_replace('/\.gif$/i', '_.gif', $icon)).' ';
	}
	
	/**
	 * Return the "toggle visibility" button
	 * @param array
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string
	 */
	public function toggleIcon($row, $href, $label, $title, $icon, $attributes)
	{
		$objInput = \Input::getInstance();
		if (strlen($objInput->get('tid')))
		{
			$this->toggleVisibility($objInput->get('tid'), ($objInput->get('state') == 1));
			$this->redirect($this->getReferer());
		}

		// Check permissions AFTER checking the tid, so hacking attempts are logged
		if (!$this->User->isAdmin && !$this->User->hasAccess('tl_revolutionslider_slides::published', 'alexf'))
		{
			return '';
		}

		$href .= '&amp;tid='.$row['id'].'&amp;state='.($row['published'] ? '' : 1);

		if (!$row['published'])
		{
			$icon = 'invisible.gif';
		}

		return '<a href="'.$this->addToUrl($href).'" title="'.specialchars($title).'"'.$attributes.'>'.$this->generateImage($icon, $label).'</a> ';
	}
	
	/**
	 * Disable/enable a user group
	 * @param integer
	 * @param boolean
	 */
	public function toggleVisibility($intId, $blnVisible)
	{
		// Check permissions to edit
		$objInput = \Input::getInstance();
		$objInput->setGet('id', $intId);
		$objInput->setGet('act', 'toggle');
		$this->checkPermission();

		// Check permissions to publish
		if (!$this->User->isAdmin && !$this->User->hasAccess('tl_revolutionslider_slides::published', 'alexf'))
		{
			$this->log('Not enough permissions to publish/unpublish item ID "'.$intId.'"', 'tl_revolutionslider_slides toggleVisibility', TL_ERROR);
			$this->redirect('contao/main.php?act=error');
		}

		$objVersions = new \Versions('tl_revolutionslider_slides', $intId);
		$objVersions->initialize();
		
		// Trigger the save_callback
		if (is_array($GLOBALS['TL_DCA']['tl_revolutionslider_slides']['fields']['published']['save_callback']))
		{
			foreach ($GLOBALS['TL_DCA']['tl_revolutionslider_slides']['fields']['published']['save_callback'] as $callback)
			{
				$this->import($callback[0]);
				$blnVisible = $this->$callback[0]->$callback[1]($blnVisible, $this);
			}
		}
		
		// Update the database
		\Database::getInstance()->prepare("UPDATE tl_revolutionslider_slides SET tstamp=". time() .", published='" . ($blnVisible ? 1 : '') . "' WHERE id=?")->execute($intId);

		$objVersions->create();
		
		$this->log('A new version of record "tl_revolutionslider_slides.id='.$intId.'" has been created', 'tl_revolutionslider_slides toggleVisibility()', TL_GENERAL);

	}
	
	/**
	 * Return the link picker wizard
	 * @param DataContainer
	 * @return string
	 */
	public function pagePicker(\DataContainer $objDC)
	{
		return ' <a href="contao/page.php?do=' . \Input::get('do') . '&amp;table=' . $objDC->table . '&amp;field=' . $objDC->field . '&amp;value=' . str_replace(array('{{link_url::', '}}'), '', $objDC->value) . '" title="' . specialchars($GLOBALS['TL_LANG']['MSC']['pagepicker']) . '" onclick="Backend.getScrollOffset();Backend.openModalSelector({\'width\':765,\'title\':\'' . specialchars(str_replace("'", "\\'", $GLOBALS['TL_LANG']['MOD']['page'][0])) . '\',\'url\':this.href,\'id\':\'' .$objDC->field . '\',\'tag\':\'ctrl_'. $objDC->field . ((\Input::get('act') == 'editAll') ? '_' . $objDC->id : '') . '\',\'self\':this});return false">' . \Image::getHtml('pickpage.gif', $GLOBALS['TL_LANG']['MSC']['pagepicker'], 'style="vertical-align:top;cursor:pointer"') . '</a>';
	}
}
