<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Core;

/**
 * Imports
 */
use RevolutionSlider\Core\RevolutionSlider as Slider;

/**
 * Class file
 * RevolutionSlider Factory
 * Provides methods to generate a new slider instance
 */
class Factory
{
	/**
	 * Return Slider object by id
	 * @param integer
	 * @param boolean
	 * @return object	RevolutionSlider
	 */
	public static function findById($intId,$bolFindSlides=true)
	{
		$objDatabase = \Database::getInstance();
		$objDbSlider = $objDatabase->prepare("SELECT * FROM tl_revolutionslider WHERE id=?")->limit(1)->execute($intId);
		
		if($objDbSlider->numRows < 1) {	return null; }
		
		// create new Revolution Slider instance
		$objRevoSlider = new Slider($objDbSlider->row());
		
		// generate slides
		if($bolFindSlides)
		{
			$objRevoSlider->findSlides($intId);
		}
			
		return $objRevoSlider;
	}
	
	/**
	 * Shortcut
	 * @reference findById()
	 */
	public static function byId($intId,$bolFindSlides=true)
	{
		return self::findById($intId,$bolFindSlides);
	}
	
	/**
	 * Return all slider ids as array
	 * @return array
	 */
	public static function getAllSliders()
	{
		$objDatabase = \Database::getInstance();
		$objResult = $objDatabase->execute("SELECT * FROM tl_revolutionslider");
						
		if($objResult->numRows < 1)
		{
			return null;
		}
		
		return $objResult->fetchEach('id');
	}

}