<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Core;

/**
 * Imports
 */
use \RevolutionSlider\Core\SlideFactory as SlideFactory;
use \RevolutionSlider\Core\Hooks as Hooks;

/**
 * Class file
 * RevolutionSlider Core Class
 * Provide methods to initialize a slider and add necessary scripts to the page
 */
class RevolutionSlider extends \Frontend
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'revoslider_default';
	
	/**
	 * Data Array
	 * @var array
	 */
	protected $arrData = array();
	
	/**
	 * Store information about parent element (cte, mod, ...)
	 * @var array
	 */
	protected $arrParent = array();

	/**
	 * Slides array
	 * @var array
	 */
	protected $arrSlides = array();

	/**
	 * Create new instance
	 * @param array
	 */
	public function __construct($arrData=array())
	{
		if(!empty($arrData))
		{
			$this->arrData = $arrData;
		}
	}

	/**
	 * Set Data
	 * @param array
	 */
	public function setData($arrData)
	{
		$this->arrData = $arrData;
	}

	/**
	 * Getters
	 * @param string
	 */
	public function get($strKey)
	{
		if(isset($this->$strKey))
		{
			return $this->$strKey;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			return $this->arrData[$strKey];
		}
		else
		{
			return null;
		}
		
	}
	
	/**
	 * Setters
	 * @param string
	 * @param mixed
	 */
	public function set($strKey, $varValue)
	{
		if(isset($this->$strKey))
		{
			$this->$strKey = $varValue;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			$this->arrData[$strKey] = $varValue;
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Generate a slider and return html and javascript
	 * @return string	html
	 */
	public function render()
	{
		// slider template
		if($this->arrData['sliderTemplate'] != $this->strTemplate)
		{
			$this->strTemplate = $this->arrData['sliderTemplate'];
		}
		$objTemplate = new \FrontendTemplate($this->strTemplate);
		$objTemplate->setData($this->arrData);
		
		$arrCssId = deserialize($this->get('cssID'));
		$arrClass = explode(' ', $arrCssId[1]);
		$arrClass[] = $this->get('sliderStyle');
		
		//-- generate an unique css class or use ID as jquery selector
		$strSelector = '';
		if(strlen($arrCssId[0]) > 0)
		{
			$objTemplate->sliderID = 'id="'.$arrCssId[0].'"';
			$strSelector = '#'.$arrCssId[0];
		}
		else
		{
			$arrParent = $this->get('arrParent');
			$elemId = $arrParent['id'];
			$elemPid = $arrParent['pid'];
			
			if($this->arrData['source'] == 'tl_content') 
			{
				$strSelector .= 'ce_revolutionslider_'.$elemId;
			}
			else if($this->arrData['source'] == 'tl_module') 
			{
				$strSelector .= 'mod_revolutionslider'.$elemId;
			}
			else {$strSelector .= 'my_revolutionslider_'.time();}
			
			$arrClass[] = $strSelector;
			$strSelector = '.'.$strSelector;
		}
		//--
		
		//-- slides
		$arrSlides = array();
		if(count($this->arrSlides) > 0 && !empty($this->arrSlides))
		{
			foreach($this->arrSlides as $i => $objSlide)
			{
				$arrRow = $objSlide->get('arrData');
				
				// classes
				$class =array('slide','slide_'.$i);
				$i%2 == 0 ? $class[] = 'even' : $arrClass[] = 'odd';
				if($i == 0) $class[] = 'first';
				if($i == count($this->arrSlides) - 1) $class[] = 'last';
				
				$cssID = deserialize($objSlide->get('cssID'));
				$class = array_merge(explode(' ', $cssID[1]),$class);
				
				// slide <li> attributes
				$arrAttributes = array();
				if(count($objSlide->get('arrAttributes')) > 0)
				{
					foreach($objSlide->get('arrAttributes') as $attr => $value)
					{
						$value = $this->replaceInsertTags($value);
						$arrAttributes[] = $attr.'="'.$value.'"';
					}
				}
				
				//-- content	
				$arrContent = $objSlide->get('arrContent');
				
				// inject main slide image on first position
				if($arrRow['lazyload'])
				{
					$dummy = 'system/modules/revolutionslider/assets/img/dummy.png';
					$dummy = $this->replaceInsertTags('{{image::'.$dummy.'?class=slideimage}}');
					
					$size = deserialize($this->get('sliderSize'));
					$file = '';
					if(version_compare(VERSION, '3.2', '<'))
					{
						$file = \FilesModel::findByPk($objSlide->get('singleSRC'))->path;
					}
					else
					{
						$file = \FilesModel::findByUuid($objSlide->get('singleSRC'))->path;
					}
					
					$image = $this->getImage($file, $size[0], $size[1], 'center_center');
					
					$image = str_replace('>',' data-lazyload="'.$image.'">', $dummy);
				}
				else
				{
					$file = \FilesModel::findByPk($objSlide->get('singleSRC'))->path;
					$image = $this->replaceInsertTags('{{image::'.$file.'?class=slideimage}}');
				}
				array_insert($arrContent,0,$image);
				//--
				
				$arrSlides[$i]['content'] 		= $arrContent;
				$arrSlides[$i]['attributes']	= implode(' ', $arrAttributes);
				$arrSlides[$i]['class']			= implode(' ', $class);
				$arrSlides[$i]['cssID']			= (strlen($cssID[0]) ? 'id="'.$cssID[0].'"' : '');
			}
		}
		
		// call hook
		$arrSlides = Hooks::getInstance()->callAddSlideHook($arrSlides,$this);
		if(empty($arrSlides))
		{
			$arrClass[] = 'empty';
			// add status messages
			$objTemplate->emptyMsg = $GLOBALS['TL_LANG']['REVOLUTIONSLIDER']['empty'];
			
			return $objTemplate->parse();
		}
		
		$objTemplate->slides = $arrSlides;
		//--
		
		//-- js template
		$objJsTemplate = new \FrontendTemplate($this->arrData['jsTemplate']);
		$objJsTemplate->setData($this->arrData);
		$objJsTemplate->selector = $strSelector;
		$objJsTemplate->stopOnHover = ($this->get('stopOnHover') ? 'on' : 'off');
		$objJsTemplate->delay = ($this->arrData['delay'] ? $this->arrData['delay'] * 1000 : 0);
		
		$arrSize = deserialize($this->arrData['sliderSize']);
		$objJsTemplate->startWidth = ($arrSize[0] ? $arrSize[0] : 0);
		$objJsTemplate->startHeight = ($arrSize[1] ? $arrSize[1] : 0);
		
		// navigation type
		if(!$this->get('addNavigation'))
		{
			$objJsTemplate->navigationType = 'none';
		}
		
		// thumbnails
		if($this->get('addThumbs'))
		{
			// enable thumbs even if the user wants no navigation but checked the addThumbs checkbox
			$objJsTemplate->navigationType = 'thumb';
			
			$arrSize = deserialize($this->get('thumbSize'));
			$objJsTemplate->thumbWidth = ($arrSize[0] ? $arrSize[0] : 0);
			$objJsTemplate->thumbHeight = ($arrSize[1] ? $arrSize[1] : 0);
		}
		
		// arrow navigation
		if(!$this->get('addArrows'))
		{
			$objJsTemplate->arrowStyle = 'none';
		}
	
		// add rs css
		$GLOBALS['TL_CSS'][] = $GLOBALS['REVOLUTIONSLIDER']['rs_css'];
		
		// add rs script
		$objJsTemplate->script = $GLOBALS['REVOLUTIONSLIDER']['rs_scriptpath'];
		$GLOBALS['TL_JAVASCRIPT']['revolutionslider'] = $GLOBALS['REVOLUTIONSLIDER']['rs_js'];
		
		// parse js template
		$strJsTemplate = $objJsTemplate->parse();
		
		// Minify inline scripts	
		if(!$GLOBALS['TL_CONFIG']['minifyMarkup'] && $GLOBALS['REVOLUTIONSLIDER']['minifyScript'])
		{
			$strChunk = $strJsTemplate;
			$strChunk = str_replace(array("/* <![CDATA[ */\n", "<!--\n", "\n//-->"), array('/* <![CDATA[ */', '', ''), $strChunk);
			$strChunk = preg_replace(array('@(?<!:)//(?!W3C|DTD|EN).*@', '/[ \n\t]*(;|=|\{|\}|\[|\]|&&|,|<|>|\',|",|\':|":|: |\|\|)[ \n\t]*/'), array('', '$1'), $strChunk);
			$strChunk = trim($strChunk);
			$strJsTemplate = $strChunk;
		}
		
		$GLOBALS['TL_HEAD'][] = $strJsTemplate;
		//--
		
		// add preset css
		if(!$GLOBALS['REVOLUTIONSLIDER']['isBundledVersion'])
		{
			$GLOBALS['TL_CSS']['revolutionslider'] = $GLOBALS['REVOLUTIONSLIDER']['css'];
		}
		
		// add classes
		$objTemplate->class = implode(' ', $arrClass);
		
		// template vars
		$arrSize = deserialize($this->arrData['sliderSize']);
		$objTemplate->startWidth = $arrSize[0];
		$objTemplate->startHeight = $arrSize[1];
		
		return $objTemplate->parse();
	}
	
	/**
	 * Set Template
	 * @param string
	 */
	public function setTemplate($strTemplate)
	{
		$this->strTemplate = $strTemplate;
	}
	
	/**
	 * Find and set all slide objects for this slider and return as array
	 * @param integer
	 * @return array
	 */
	public function findSlides($intId)
	{
		$this->arrSlides = SlideFactory::findAllBySlider($intId);
		
		return $arrSlides;
	}
	
	/**
	 * Return all transition options as array
	 * @return array
	 */
	public function getTransitions()
	{
		$arrReturn = array(
			'boxslide','boxfade', 'slotzoom-horizontal', 'slotzoom-vertical',
			'slotslide-horizontal', 'slotslide-vertical', 'slotfade-horizontal',
			'slotfade-vertical', 'curtain-1', 'curtain-2', 'curtain-3',
			'slideleft', 'slideright', 'slideup' ,'slidedown',
			'slidehorizontal', 'slidevertical',	'fade',	'random'
		);

		// premiums
		$arrPremiums = array
		(
			 'papercut','flyin','turnoff','cube','3dcurtain-vertical',
			 '3dcurtain-horizontal','premium-random'
		);
		
		// HOOK allow more transitions via global var
		if(count($GLOBALS['REVOLUTIONSLIDER']['TRANSITIONS']) > 0)
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['TRANSITIONS']);
		}
		
		$arrReturn = array_merge($arrReturn, $arrPremiums);

		// order
		ksort($arrReturn);

		return $arrReturn;
	}
	
	/**
	 * Return easing options as array
	 * @return array
	 */	
	public function getTransitionEasings()
	{
		$arrReturn = array
		(
			 'easeInBack','easeOutBack', 'easeOutBack', 'easeInOutBack',
			 'easeInQuad', 'easeOutQuad', 'easeInOutQuad',  
			 'easeOutQuart', 'easeInOutQuart',
			 'easeInCubic', 'easeOutCubic', 'easeInOutCubic', 'easeInQuart',
			 'easeInQuint', 'easeOutQuint', 'easeInOutQuint',
			 'easeInSine', 'easeOutSine', 'easeInOutSine', 
			 'easeInExpo', 'easeOutExpo', 'easeInOutExpo',
			 'easeInCirc', 'easeOutCirc', 'easeInOutCirc', 
			 'easeInElastic', 'easeOutElastic','easeInOutElastic',  
			 'easeInBounce', 'easeOutBounce', 'easeInOutBounce',
		);
		
		// HOOK allow more easings via global var
		if(count($GLOBALS['REVOLUTIONSLIDER']['EASINGS']) > 0)
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['EASINGS']);
		}
		
		return $arrReturn;
	}
	
	/**
	 * Return start animations classes as array
	 * @return array
	 */	
	public function getStartAnimationClasses()
	{
		$arrReturn = array
		(
			'sft',	//- Short from Top
			'sfb',	// - Short from Bottom
			'sfr',	// - Short from Right
			'sfl',	// - Short from Left
			'lft',	// - Long from Top
			'lfb',	// - Long from Bottom
			'lfr',	// - Long from Right
			'lfl',	// - Long from Left
			'fade',	// - fading
			'randomrotate',	//- Fade in, Rotate from a Random position and Degree
		);
		
		// HOOK allow more animation classes via global var
		if(count($GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']) > 0)
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']);
		}
		
		return $arrReturn;
	}
	
	/**
	 * Return end animations classes as array
	 * @return array
	 */	
	public function getEndAnimationClasses()
	{
		$arrReturn = array
		(
			'stt',	// - Short to Top
			'stb',	// - Short to Bottom
			'str',	// - Short to Right
			'stl',	// - Short to Left
			'ltt',	// - Long to Top
			'ltb',	// - Long to Bottom
			'ltr',	// - Long to Right
			'ltl',	//- Long to Left
			'fade',	// - fading
			'randomrotate',	//- Fade in, Rotate from a Random position and Degree
		);
		
		// HOOK allow more animation classes via global var
		if(count($GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']) > 0)
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']);
		}
		
		return $arrReturn;
	}

}