<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 * @license		http://www.gnu.org/licenses/lgpl-3.0.html LGPL
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Core;

/**
 * Class files
 * RevolutionSliderSlide
 */
class Slide extends \Controller
{
	/**
	 * Content elements array
	 * @var array
	 */
	protected $arrContent = array();
	
	/**
	 * Attributes array
	 * @var array
	 */
	protected $arrAttributes = array();
	
	/**
	 * Data Array
	 * @var array
	 */
	protected $arrData = array();

	/**
	 * Create new instance
	 * @param array
	 */ 
	public function __construct($arrData)
	{
		$this->arrData = $arrData;
		$this->arrContent = $this->prepareContent($arrData['id']);
		$this->arrAttributes = array
		(
			'data-transition'	=> $arrData['transition'],
			'data-slotamount'	=> $arrData['slotamount'],
		);
		
		// delay
		if($arrData['delay'] > 0)
		{
			$this->arrAttributes['data-delay'] = $arrData['delay'] * 1000;
		}
		
		// masterspeed
		if($arrData['masterspeed'] > 0)
		{
			$this->arrAttributes['data-masterspeed'] = $arrData['delay'] * 1000;
		}
		
		// link slide
		if(strlen($arrData['slideUrl']) > 0)
		{
			$this->arrAttributes['data-link'] = $arrData['slideUrl'];
			$this->arrAttributes['data-target'] = ($arrData['newWindow'] ? '_blank' : '');
		}
		
		// generate thumbs
		$objSlider = $this->getSlider();
		if($objSlider->get('addThumbs'))
		{
			$this->arrAttributes['data-thumb'] = $this->getThumb();
		}
	}
	
	/**
	 * Set Data
	 * @param array
	 */
	public function setData($arrData)
	{
		$this->arrData = $arrData;
	}

	/**
	 * Getters
	 * @param string
	 */
	public function get($strKey)
	{
		if(isset($this->$strKey))
		{
			return $this->$strKey;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			return $this->arrData[$strKey];
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Setters
	 * @param string
	 * @param mixed
	 */
	public function set($strKey, $varValue)
	{
		if(isset($this->$strKey))
		{
			$this->$strKey = $varValue;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			$this->arrData[$strKey] = $varValue;
		}
		else
		{
			return null;
		}
	}
		
	/**
	 * Prepare the slide
	 * Fetch all content elements
	 * @param array
	 * @return array
	 */
	public function prepareContent($intSlide)
	{
		// get raw content data 
		$arrRawContent = $this->findContentBySlide($intSlide);
		
		if(count($arrRawContent) < 1 || empty($arrRawContent))
		{
			return array();
		} 
		
		// generate content
		$arrContent = array();
		foreach($arrRawContent as $cte)
		{
			// attributes
			$data_postion = deserialize($cte['revolutionslider_data_position']);
			$arrAttributes = array
			(
				'data-easing'	=> $cte['revolutionslider_data_easing'],
				'data-speed'	=> $cte['revolutionslider_data_speed'] * 1000, // sec to ms
				'data-x'		=> (strlen($data_postion[0]) ?  $data_postion[0]: '0'),
				'data-y'		=> (strlen($data_postion[1]) ?  $data_postion[1]: '0'),
				'data-start'	=> $cte['revolutionslider_data_start'] * 1000,
			);
			
			// link element to slide
			if($cte['revolutionslider_data_linktoslide'] > 0)
			{
				$arrAttributes['data-linktoslide'] = $cte['revolutionslider_data_linktoslide'];
			}
			
			if($cte['type'] == 'revolutionslider_video')
			{
				$arrAttributes['data-autoplay'] 		= ($cte['revolutionslider_data_autoplay'] ? 'true' : 'false');
				$arrAttributes['data-nextslideatend']	= ($cte['revolutionslider_data_nextslideatend'] ? 'true' : 'false');
			}
			
			// generate the content element
			$strCte = $this->replaceInsertTags('{{insert_content::'.$cte['id'].'}}');
			
			// wrap the content element in a container div
			#$strCte = '<div class="tp-caption">'.$strCte.'</div>';
			
			// inject attributes in content element
			$strAttributes = '';
			foreach($arrAttributes as $attr => $value)
			{
				$strAttributes .= $attr .'="'.$value.'" ';
			}
			trim($strAttributes);
			
			// since we only work on html elements we can easily just replace everything in the first <...>
			$preg = preg_match('/<(.*?)\>/', $strCte,$result);
			if($preg)
			{
				$search = $result[0];
				$replace = sprintf('<%s %s>',$result[1], $strAttributes);
				$strCte = str_replace($search, $replace, $strCte);
			}
			
			// add must have classes
			$preg = preg_match('/class="(.*?)\"/', $strCte,$result);
			if($preg)
			{
				$arrClass = explode(' ', trim($result[1]));
				
				// add class "tp-caption"
				if(!in_array('tp-caption', $arrClass))
				{
					$arrClass[] = 'tp-caption';
				}
				
				// add start animation class
				if(!in_array($cte['revolutionslider_data_animation_start'], $arrClass))
				{
					$arrClass[] = $cte['revolutionslider_data_animation_start'];
				}
				
				// add end animation class
				if(!in_array($cte['revolutionslider_data_animation_end'], $arrClass))
				{
					$arrClass[] = $cte['revolutionslider_data_animation_end'];
				}
				
				$search = $result[0];
				$strCte = str_replace($search, 'class="'.implode(' ', $arrClass).'"', $strCte);
			}
			
			$arrContent[] = $strCte;
		}
		
		return $arrContent;
	}
	 
	/**
	 * Find content elements of a slide
	 * @param integer
	 * @return array
	 */
	protected function findContentBySlide($intSlide)
	{
		$objDatabase = \Database::getInstance();
		
		$objStm = null;
		if (version_compare(VERSION, '2.11', '<='))
		{
			$objStm = $objDatabase->prepare("SELECT * FROM tl_content WHERE do='revolutionslider' AND pid=? AND invisible!=1 ORDER BY sorting");
		}
		else
		{
			$objStm = $objDatabase->prepare("SELECT * FROM tl_content WHERE ptable='tl_revolutionslider_slides' AND pid=? AND invisible!=1 ORDER BY sorting");
		}
		
		$objCte = $objStm->execute($intSlide);
		
		return $objCte->fetchAllAssoc();
	}

	/**
	 * Generate the current thumbnail from the main image or from the slide image and return the filepath
	 * @param string
	 * @param string
	 * @param string
	 * @param string
	 * @return string	path to thumb
	 */
	protected function getThumb()
	{
		if($this->get('singleSRC') == '' && $this->get('singleSRC_thumb') == '')
		{
			return '';
		}
		
		$objSlider = $this->getSlider();
		
		$src = $this->get('singleSRC_thumb');
		if($src == '')
		{
			$src = $this->get('singleSRC');
		}
		
		$size = deserialize($objSlider->get('thumbSize'));
		
		$file = '';
		if(version_compare(VERSION, '3.2', '<'))
		{
			$file = \FilesModel::findByPk($src)->path;
		}
		else
		{
			$file = \FilesModel::findByUuid($src)->path;
		}
		
		return $this->getImage($file,$size[0],$size[1],$size[2]);
	}
	
	/**
	 * Generate the video element and return html string
	 * @param array
	 * @return string	html
	 */
	protected function getContentVideo($arrData)
	{
		$strReturn = '';
		
		// get parent slider and set iframe size to slider size
		$objSlider = $this->getSlider();
		$size = deserialize($objSlider->get('sliderSize'));
		$w = ($size[0] > 0 ? $size[0] : '100%');
		$h = ($size[1] > 0 ? $size[1] : '100%');
		
		$attr = array('width="'.$w.'" height="'.$h.'"');
		
		// create an iframe for the video element from external sources	
		if($arrData['revolutionslider_videoType'] == 'external' && strlen($arrData['url']) > 0)
		{
			$attr[] = 'scrolling="no"';
			$src = $arrData['url'];
			
			$strReturn = sprintf('<div class="video"><iframe src="%s" %s></iframe></div>',$src,implode(' ',$attr));
			return $strReturn;
		}
		// local HTML5 video files
		else if($arrData['revolutionslider_videoType'] == 'local' && strlen($arrData['multiSRC']) > 0)
		{
			$arrSources = array();
			if (version_compare(VERSION, '2.11', '<='))	
			{
				$arrSources = deserialize($arrData['multiSRC']);
			}
			else
			{
				$arrFiles = deserialize($arrData['multiSRC']);
				$objFiles = \FilesModel::findMultipleByIds($arrFiles);
				if(!$objFiles)
				{
					return '';
				}
				
				$arrSources = $objFiles->fetchEach('path');
			}
			
			if(count($arrSources) < 1)
			{
				return '';
			}
			
			$sources = array();
			foreach($arrSources as $source)
			{
				$ext = pathinfo($source, PATHINFO_EXTENSION);
				
				if(in_array($ext, array('flv')) )
				{
					continue;
				}
				
				if($ext == 'ogv') {$ext = 'ogg';}
				
				$sources[] = '<source src="'.$source.'" type="video/'.$ext.'"/>';
			}
			$sources[] = '<track kind="captions" src="demo.captions.vtt" srclang="en" label="English" />';
			
			$attr[] = 'controls';
			$attr[] = 'preload="none"';
			$attr[] = 'poster="'.$strPoster.'"';
			$attr[] = 'data-setup="{'.$strDataSetup.'}"';
			
			$videoClass = array('video-js','vjs-default-skin');
			
			$strReturn = sprintf('<div class="video"><video class="%s" %s>%s</video></div>',
				implode(' ',$videoClass),
				implode(' ',$attr),
				implode(' ',$sources)
				);
			
			return $strReturn;
		}
		else
		{
			// HOOK here?
			
			return $arrReturn;
		}
			
	}

	/**
	 * Return the parent slider object
	 * Optional allow to generate the slides
	 * @return object	RevolutionSlider
	 */
	public function getSlider()
	{
		return \RevolutionSlider\Core\Factory::byId($this->get('pid'),false);
	}
	
}