<?php

/**
 * Contao Open Source CMS
 *
 * Copyright (C) 2005-2013 Leo Feyer
 *
 * @copyright Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author  Tim Gatzky <info@tim-gatzky.de>
 * @package  revolutionslider
 * @link  http://contao.org
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Frontend;

/**
 * Imports
 */
use \Database;

/**
 * Class file
 */
class ContentRevolutionSliderVideo extends \ContentElement
{
	/**
	 * Template
	 * @var
	 */
	protected $strTemplate = 'ce_revolutionslider_video';

	/**
	 * Display wildcard
	 */
	public function generate()
	{
		if (TL_MODE == 'BE')
		{
			// display video sources
			$strInfo = '';
			if($this->revolutionslider_videoType == 'external')
			{
				$strInfo = '<p class="tl_gray">'.$this->url.'</p>';
			}
			else if($this->revolutionslider_videoType == 'local' && strlen($this->multiSRC) > 0)
				{
					$arrSources = array();
					$arrFiles = deserialize($this->multiSRC);
					$objFiles = \FilesModel::findMultipleByIds($arrFiles);
					$arrSources = $objFiles->fetchEach('path');

					$strInfo .= '<p class="tl_gray">';
					foreach($arrSources as $src)
					{
						$strInfo .= '<div class="tl_gray no-padding">'.$src.'</div>';
					}
					$strInfo.='</p>';
				}

			$this->Template = new \BackendTemplate('be_wildcard');
			$this->Template->wildcard = '### REVOLUTION SLIDER VIDEO ###'. $strInfo;;
			$this->Template->title = $this->headline;

			return $this->Template->parse();
		}

		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		// get parent slider and set iframe or video size to slider size
		$objDatabase = \Database::getInstance();
		$objSlider = $objDatabase->prepare("SELECT * FROM tl_revolutionslider WHERE id=
			(SELECT pid FROM tl_revolutionslider_slides WHERE id=?)")
						->limit(1)
						->execute($this->pid);
		
		if($objSlider->numRows < 1)
		{
			return;
		}
		
		$size = deserialize($objSlider->sliderSize);
		$w = ($size[0] > 0 ? $size[0] : '100%');
		$h = ($size[1] > 0 ? $size[1] : '100%');
		
		if( in_array($objSlider->sliderStyle, array('responsive','fullwidth')) )
		{
			$w = '100%';
			$h = '100%';
		}
		
		$arrSources = array();
		if($this->revolutionslider_videoType == 'external' && strlen($this->url) > 0)
		{
			$attr[] = 'scrolling="no"';
			$src = $this->url;

			$attr = array('width="'.$w.'" height="'.$h.'"');
			$this->Template->iframe =  sprintf('<iframe src="%s" %s></iframe>',$src,implode(' ',$attr));
		}
		// local HTML5 video files
		else if($this->revolutionslider_videoType == 'local' && strlen($this->multiSRC) > 0)
		{
			$attr[] = 'controls';
			$attr[] = 'preload="none"';
			$attr[] = 'poster="'.$strPoster.'"';
			$attr[] = 'data-setup="{'.$strDataSetup.'}"';
			
			$arrFiles = deserialize($this->multiSRC);
			$objFiles = \FilesModel::findMultipleByIds($arrFiles);
			if(!$objFiles)
			{
				return '';
			}

			$arrSources = $objFiles->fetchEach('path');
		}
		else
		{
			// HOOK here?
		}
		
		$arrFiles =	array();
		foreach($arrSources as $source)
		{
			$ext = pathinfo($source, PATHINFO_EXTENSION);
				
			if(in_array($ext, array('flv')) )
			{
				continue;
			}
			
			if($ext == 'ogv') {$ext = 'ogg';}
			
			$arrFiles[] = array
			(
				'src' 	=> $source,
				'mime'	=> 'video/'.$ext,
			);
		}
		
		$arrClass = explode(' ', $this->cssID[1]);
		$arrClass[] = $this->strTemplate;
		
		if( in_array($objSlider->sliderStyle, array('responsive','fullwidth')) )
		{
			$this->Template->fullwidth = true;
			$arrClass[] = 'fullscreenvideo';
			$arrClass = array_unique($arrClass);
		}
		
		$this->Template->class = trim(implode(' ', $arrClass));
		$this->Template->attributes = trim(implode(' ', $attr));
		$this->Template->files = $arrFiles;
		$this->Template->source = $this->revolutionslider_videoType;
		
		return $this->Template->parse();
	}

	/**
	 * Find files in folders recursive
	 * @param array
	 * @param boolean
	 * @return array
	 */
	protected function findFiles($strDir,$arrFiles=array(),$bolRecursive=true)
	{
		if(!is_dir($strDir))
		{
			return array();
		}
		
		$scan = scandir($strDir);
		if(count($scan) < 0)
		{
			return array();
		}
		
		foreach($scan as $f)
		{
			if(strpos($f, '.') === 0)
			{
				continue;
			}
			
			if(is_dir($strDir.'/'.$f) && $bolRecursive)
			{
				$arrFiles = $this->findFiles($strDir.'/'.$f,$arrFiles);
				continue;
			}
			
			$arrFiles[] = $strDir.'/'.$f; 
		}
		
		return $arrFiles;	
	}
	
	
}